from Cryptodome.Cipher import AES
from Cryptodome import Random
import hashlib
from autoIserv.Session import Session


def encrypt(raw, key):
    BS = 16
    pad = lambda s: s + (BS - len(s) % BS) * bytes(chr(BS - len(s) % BS), encoding="utf-8")
    raw = pad(raw)
    iv = Random.new().read(AES.block_size)
    cipher = AES.new(key, AES.MODE_CBC, iv)
    return iv + cipher.encrypt(raw)


def decrypt(enc, key):
    BS = 16
    unpad = lambda s: s[:-ord(s[len(s) - 1:])]
    enc = enc
    iv = enc[:AES.block_size]
    cipher = AES.new(key, AES.MODE_CBC, iv)
    return unpad(cipher.decrypt(enc[AES.block_size:])).decode('utf-8')


def load_credential_file(credential_file_path : str, key : str):
    key = key.encode(encoding="utf-8")
    key = hashlib.sha256(key).digest()
    file = open(credential_file_path, mode="rb")
    username, password = decrypt(file.read(), key).split("\n")
    password = password.replace("\n", "")
    file.close()
    return username, password


def create_credential_file(username : str, password : str, key : str, credential_file_path : str):
    key = key.encode(encoding="utf-8")
    key = hashlib.sha256(key).digest()
    file = open(credential_file_path, mode="wb")
    file.write(encrypt(bytes(username, encoding="utf-8") + bytes("\n", encoding="utf-8") + bytes(password, encoding="utf-8"), key))
    file.close()


def login(credential_file_path : str, key : str, *args, **kwargs) -> Session:
    print("[*] Loading credentials")
    username, password = load_credential_file(credential_file_path, key)
    session = Session(username, password, *args, **kwargs)
    return session



import autoIserv
import getpass
import time

def main():
    COLOR_CYAN = str(b'\x1b[36m', encoding="utf-8")
    COLOR_GREEN = str(b'\x1b[32m', encoding="utf-8")
    COLOR_YELLOW = str(b'\x1b[33m', encoding="utf-8")
    COLOR_RED = str(b'\x1b[31m', encoding="utf-8")
    COLOR_RESET = str(b'\x1b[0m', encoding="utf-8")

    print(f"{COLOR_CYAN}---------Credential creation---------{COLOR_RESET}")
    username = input(f"{COLOR_GREEN}Enter Iserv username : {COLOR_RESET}")
    while True:
        password = getpass.getpass(f"{COLOR_YELLOW}Enter Iserv password : {COLOR_RESET}")
        password_again = getpass.getpass(f"{COLOR_YELLOW}Enter Iserv password again: {COLOR_RESET}")
        if password != password_again:
            print(f"{COLOR_RED}[*] Passwords are not equal{COLOR_RESET}")
        else:
            break

    while True:
        key = getpass.getpass(f"{COLOR_RED}Enter secret credential encryption key : {COLOR_RESET}")
        key_again = getpass.getpass(f"{COLOR_RED}Enter secret credential encryption key : {COLOR_RESET}")
        if key != key_again:
            print(f"{COLOR_RED}[*] Keys are not equal{COLOR_RESET}")
        else:
            break
    print(f"{COLOR_CYAN}-------------------------------------{COLOR_RESET}")

    print(f"{COLOR_CYAN}[*] Creating Credentials{COLOR_RESET}")
    autoIserv.create_credential_file(username, password, key, "credential")
    time.sleep(0.3)
    print(f"{COLOR_CYAN}[*] Done{COLOR_RESET}")
    input(f"{COLOR_CYAN}Press Enter to exit{COLOR_RESET}")


if __name__ == "__main__":
    main()


from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By

from autoIserv import Session

import time

class Module:

    def __init__(self, module_name, session: Session):
        self.module_name = module_name
        self.session = session
        # Contains content of the current module not the sidebar
        self.content = None # type: WebElement


    def navigate(self):
        self.session.open_module(self.module_name)
        time.sleep(1)  # Don't know why but it works with this
        self.set_content()


    def set_content(self):
        self.content = WebDriverWait(self.session.driver, 3).until(EC.presence_of_element_located((By.ID, "content")))



from .. import Session, util
from ..Module import Module

from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.common.exceptions import NoSuchElementException, TimeoutException, StaleElementReferenceException
from selenium.webdriver.common.by import By
import time
import datetime
import pathlib


class FilesModule(Module):

    def __init__(self, session: Session):
        super().__init__("Dateien", session)


    def get_nav_bar_folder_elements(self):
        nav_bar = WebDriverWait(self.content, 3).until(EC.presence_of_element_located((By.CLASS_NAME, "navbar-nav")))  # type: WebElement
        nav_bar_file_links = nav_bar.find_elements_by_class_name("files-link")
        return nav_bar_file_links


    def change_to_nav_bar_folder(self, name: str):
        for folder_element in self.get_nav_bar_folder_elements():
            if folder_element.text == name:
                self.change_to_folder(folder_element.get_attribute("href"))


    def get_current_dir_files_and_dirs(self, ignore_size=False):
        file_table = self.content.find_element_by_id("files").find_element_by_tag_name("tbody")

        ignored_exceptions = (NoSuchElementException, StaleElementReferenceException,)
        rows = WebDriverWait(file_table, 3, ignored_exceptions=ignored_exceptions).until(EC.presence_of_all_elements_located((By.TAG_NAME, "tr")))

        for row in rows: # type: WebElement
            file = File.from_element(row, ignore_size)
            if file != None:
                yield file


    def change_to_folder(self, remote_location: str):
        self.session.driver.get(remote_location)
        time.sleep(1)
        self.set_content()


class File:

    FILE_LIST_COLUMN_DATA_TYPE_ORDER = ["id", "name", "size", "type", "owner", "last_change"]

    def __init__(self):
        self.id = ""
        self.name = ""
        self.size = 0
        self.type = ""
        self.owner = ""
        self.remote_location = ""
        self.last_change = None # type: datetime.datetime


    @staticmethod
    def from_element(element: WebElement, ignore_size=False):
        try:
            ignored_exceptions = (NoSuchElementException, StaleElementReferenceException,)
            WebDriverWait(element, 3, ignored_exceptions=ignored_exceptions).until(EC.presence_of_all_elements_located((By.CLASS_NAME, "files-id")))  # type: WebElement
        except TimeoutException:
            return None
        file = File()
        for i, col in enumerate(element.find_elements_by_tag_name("td")):
            if i >= len(File.FILE_LIST_COLUMN_DATA_TYPE_ORDER):
                continue
            data_type = File.FILE_LIST_COLUMN_DATA_TYPE_ORDER[i]
            if data_type == "id":
                file.id = col.find_element_by_tag_name("input").get_attribute("value")
            elif data_type == "name":
                file.name = col.text
                file.remote_location = col.find_element_by_tag_name("a").get_attribute("href")
            elif data_type == "size":
                try:
                    calc_size_button = col.find_element_by_tag_name("button")
                    if not ignore_size:
                        calc_size_button.click()
                        while col.text == "berechnen":
                            time.sleep(0.1)
                        file.parse_size(col.text)
                except NoSuchElementException:
                    file.parse_size(col.text)
            elif data_type == "type":
                file.type = col.text
            elif data_type == "owner":
                file.owner = col.text
            elif data_type == "last_change":
                file.parse_last_change(col.text)
        return file


    def parse_size(self, size_text: str):
        size_text = size_text.replace(" KB", "")
        size_text = size_text.replace(".", "")
        size_text = size_text.replace(",", "")
        if size_text == "":
            time.sleep(1)
            self.size = 0
            return
        self.size = int(size_text)


    def parse_last_change(self, last_change_text):
        self.last_change = datetime.datetime.strptime(last_change_text, "%d.%m.%Y %H:%M")


    def navigate(self, session: Session):
        session.driver.get(self.remote_location)


    def download(self, session: Session, save_to: pathlib.Path):
        if self.type in ["Datei", "Archiv"]:
            _, content = util.download_file_from_iserv(session.driver, self.remote_location)
            f = open(str(save_to.joinpath(self.name)), mode="wb")
            f.write(content)
            f.close()
        else:
            raise IsADirectoryError("Folders can not be downloaded")


    def __repr__(self):
        return f"<File name={self.name} type={self.type} size={self.size} owner={self.owner} last_change={self.last_change} remote_location={self.remote_location}>"


import requests


def download_file_from_iserv(driver, download_url):
    cookies = driver.get_cookies()
    real_cookies = dict()
    for cookie in cookies:
        if cookie["secure"] == True:
            real_cookies[cookie["name"]] = cookie["value"]

    result = requests.get(download_url, cookies=real_cookies, allow_redirects=True)
    try:
        filename = result.headers.get("Content-Disposition").split(";")[1].split("=")[-1]
        if filename:
            return filename, result.content
    except:
        return None, None


from autoIserv.util import download_file_from_iserv

from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.firefox.webelement import FirefoxWebElement
from selenium.webdriver.firefox.webdriver import WebDriver
from selenium.common.exceptions import NoSuchElementException, TimeoutException
from urllib.parse import urljoin
import os
import json


class Exercise:

    SUBJECT_KEYWORDS = {
        "Latin": ["Latein"],
        "France": ["Französisch"],
        "Spanish": ["Spanisch"],
        "Biology" : ["Biologie", "Bio"],
        "Physics": ["Physik"],
        "Chemistry": ["Chemie"],
        "Music" : ["Musik"],
        "Art" : ["Kunst"],
        "Computer Science" : ["Informatik"],
        "History": ["Geschichte", "Geschichtsaufgabe"],
        "Politics" : ["Politik", "Gesetze"],
        "English": ["Englisch", "English", "Unit"],
        "Math": ["Matheaufgaben", "Mathe", "Mathematik", "Berechne", "Berechnen"],
        "German": ["Deutsch", "Gedicht"],
    }

    def __init__(self):
        self.html_element = None
        self.index = None
        self.driver = None
        self.real_base_url = None

        # Basic Data
        self.title = None
        self.start_date = None
        self.due_date = None
        self.subject = None
        self.view_url = None
        self.tags = None

        # Advanced Data
        self.description = None
        self.by = None
        self.attachments = {}

        self.new = True

    def get_basic_data(self, html_element: FirefoxWebElement, index: int, web_driver: WebDriver, base_url: str):
        self.html_element = html_element
        self.index = index
        self.driver = web_driver
        self.real_base_url = base_url.split("/")[0]

        columns = html_element.find_elements_by_tag_name("td")
        for i, column in enumerate(columns):
            if i == 0:
                self.title = column.text
                try:
                    show_link_element = column.find_element_by_tag_name("a")
                    self.view_url = show_link_element.get_attribute("href")
                except NoSuchElementException:
                    return False
            elif i == 1:
                self.start_date = datetime.strptime(column.text, "%d.%m.%Y")
            elif i == 2:
                span_element = column.find_element_by_tag_name("span")
                due_date_data_string = span_element.get_attribute("data-date").split("+")[0]
                self.due_date = datetime.strptime(due_date_data_string, "%Y-%m-%dT%H:%M:%S")
            elif i == 3:
                tags_text = column.text
                if tags_text != "(keine)":
                    self.tags = str(column.text)
                    self.figure_out_subject_based_on_tags()
        return True


    def figure_out_subject(self):
        if self.subject != "not_found" and self.subject != None:
            return
        search_string = (str(self.title) + str(self.description)).lower()
        for subject in self.SUBJECT_KEYWORDS:
            for key_word in self.SUBJECT_KEYWORDS[subject]:
                if key_word.lower() in search_string:
                    self.subject = subject
                    return
        self.subject = "not_found"


    def figure_out_subject_based_on_tags(self):
        if self.tags:
            for subject in self.SUBJECT_KEYWORDS:
                for key_word in self.SUBJECT_KEYWORDS[subject]:
                    if key_word.lower() in self.tags.lower():
                        self.subject = subject
                        return
        self.subject = "not_found"


    def view(self, download_attachments=True):
        self.driver.get(self.view_url)
        print(f"[*] Viewing Exercise : '{self.title}'")
        site_content = WebDriverWait(self.driver, 5).until(EC.presence_of_element_located((By.CLASS_NAME, "page-content")))
        site_row = site_content.find_element_by_class_name("row")
        information_table = WebDriverWait(site_row, 3).until(EC.presence_of_element_located((By.XPATH, "div[1]/div/table/tbody")))
        information_rows = information_table.find_elements_by_tag_name("tr")

        for row in information_rows:
            try:
                header = row.find_element_by_tag_name("th").text
                data = row.find_element_by_tag_name("td")
                if "Erstellt von" in header:
                    self.by = data.text
                elif "Beschreibung" in header:  # Description is not in this table anymore but you can never be sure enough.
                    self.description = data.text
            except NoSuchElementException:
                pass

        if self.description == None:
            # Get description text from div below information table
            description_div = WebDriverWait(site_row, 3).until(EC.presence_of_element_located((By.XPATH, "div[1]/div/div[2]/div[2]")))
            self.description = description_div.text

        if download_attachments:
            try:
                attachment_table_body = WebDriverWait(site_row, 1).until(EC.presence_of_element_located((By.XPATH, "div[1]/div/form/table/tbody")))
                attachment_rows = attachment_table_body.find_elements_by_tag_name("tr")
                for attachment_row in attachment_rows:
                    link_element = attachment_row.find_element_by_tag_name("a")

                    download_url = urljoin(self.real_base_url, link_element.get_attribute('href'))
                    print(f"[*] Downloading Attachment : {download_url}")
                    result = download_file_from_iserv(self.driver, download_url)
                    if result:
                        print(f"[*] '{result[0]}' was downloaded")
                        self.attachments[result[0]] = {
                            "content": result[1],
                            "description": link_element.text
                        }
            except TimeoutException: # No attachments for that exercises exist
                pass
        self.figure_out_subject()


    def save(self, exercises_dir, attachment_dir):
        data = {
            "Title": self.title,
            "Description": self.description,
            "By": self.by,
            "Tags" : self.tags,
            "ViewUrl" : self.view_url,
            "StartDate": self.start_date.strftime("%d.%m.%Y %H_%M_%S"),
            "DueDate": self.due_date.strftime("%d.%m.%Y %H_%M_%S"),
            "Attachments": [],
            "New" : self.new
        }

        if self.attachments != None:
            for attachment_name in self.attachments:
                attachment = self.attachments[attachment_name]
                attachment_content = attachment["content"]
                attachment_description = attachment["description"]
                data["Attachments"].append({
                    "Name" : attachment_name,
                    "Description" : attachment_description
                })
                if attachment_dir:
                    if os.path.isdir(attachment_dir):
                        file_path = f"{attachment_dir}\\{attachment_name}"
                        print(f"[*] Saving Attachment to {file_path}")
                        file = open(file_path, mode="wb")
                        file.write(attachment_content)
                        file.close()


        exercise_path = f"{exercises_dir}\\" + f"{self.title}-{self.start_date}-{self.due_date}{self.by}.json".replace(":", "").replace("\\", "").replace("/", "").replace("?", "").replace("*", "").replace("<", "").replace(">","").replace('"', "").replace("|", "")
        print(f"[*] Saving Exercise to : {exercise_path}")
        with open(exercise_path, 'w', encoding="utf-8") as file:
            json.dump(data, file)


    @staticmethod
    def load(path):
        new_exercise = Exercise()
        data = json.load(open(path, mode="r", encoding="utf-8"))
        new_exercise.title = data["Title"]
        new_exercise.description = data["Description"]
        new_exercise.view_url = data["ViewUrl"]
        new_exercise.by = data["By"]
        if "Tags" in data:
            new_exercise.tags = data["Tags"]
        new_exercise.start_date = datetime.strptime(data["StartDate"], "%d.%m.%Y %H_%M_%S")
        new_exercise.due_date = datetime.strptime(data["DueDate"], "%d.%m.%Y %H_%M_%S")
        new_exercise.new = data["New"]
        new_exercise.figure_out_subject()

        try:
            attachments = data["Attachments"]
            for attachment in attachments:
                new_exercise.attachments[attachment["Name"]] = {
                    "content" : b"",
                    "description" : attachment["Description"]
                }
        except KeyError:
            new_exercise.attachments = {}

        return new_exercise


    def __eq__(self, other):
        assert isinstance(other, Exercise)
        return self.title == other.title and self.start_date == other.start_date and self.due_date == other.due_date

    def __repr__(self):
        return f"<autoIserv.Exercise.Exercise (title='{self.title}' start_date={self.start_date} due_date={self.due_date} by='{self.by}' subject={self.subject})>"




from .. import Session, Exercise
from ..Module import Module

from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By


class ExerciseModule(Module):
    def __init__(self, session: Session):
        super().__init__("Aufgaben", session)


    def get_exercises(self):
        self.navigate()
        print("[*] Getting exercise Table")
        exercise_table = WebDriverWait(self.content, 5).until(EC.presence_of_element_located((By.ID, "crud-table")))
        exercise_table_body = exercise_table.find_element_by_tag_name("tbody")
        exercise_rows = WebDriverWait(exercise_table_body, 5).until(EC.presence_of_all_elements_located((By.TAG_NAME, "tr")))
        exercises = []
        for i, exercise_element in enumerate(exercise_rows):
            exercise = Exercise()
            if exercise.get_basic_data(exercise_element, i, self.session.driver, self.session.base_url):
                exercises.append(exercise)
        return exercises


from selenium import webdriver
from selenium.common.exceptions import TimeoutException, WebDriverException
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

class Session:

    def __init__(self, username : str, password : str, headless : bool = True, iserv_url : str = "", use_chrome : bool = False):
        print("[*] Starting Webdriver")
        if not use_chrome:
            options = webdriver.FirefoxOptions()
            options.headless = headless
            profile = webdriver.FirefoxProfile()
            self.driver = webdriver.Firefox(firefox_options=options, firefox_profile=profile)
        else:
            print("Warning : Chrome is not relay supported and is not tested.")
            self.driver = webdriver.Chrome()
        self.base_url = iserv_url
        try:
            self.login(username, password)
        except WebDriverException as e:
            self.close()
            raise e


    def login(self, username : str, password : str):
        print("[*] Navigating to Login Page")
        self.driver.get(f"{self.base_url}/login")
        username_input = WebDriverWait(self.driver, 3).until(EC.presence_of_element_located((By.XPATH,"//input[@placeholder='Account']")))
        password_input = WebDriverWait(self.driver, 3).until(EC.presence_of_element_located((By.XPATH,"//input[@placeholder='Passwort']")))
        username_input.send_keys(username)
        password_input.send_keys(password)
        print("[*] Submitting credentials")
        login_button = WebDriverWait(self.driver, 3).until(EC.presence_of_element_located((By.XPATH,"//button[@type='submit']")))
        login_button.click()

        try:
            WebDriverWait(self.driver, 5).until(EC.presence_of_element_located((By.XPATH, "//li[@class='sidebar-profile']")))
        except TimeoutException:
            raise ValueError("Wrong credentials")

        print("[*] Right credentials")


    def open_module(self, module_name : str):
        modules = self.driver.find_elements_by_class_name("nav-module")
        for module in modules:
            label = module.find_elements_by_class_name("item-label")[0]
            current_module_name = label.get_attribute('innerHTML')
            if current_module_name == module_name:
                link = module.find_element_by_tag_name("a").get_attribute("href")
                print(f"[*] Navigating to : {module_name}")
                self.driver.get(link)
                return link
        raise ModuleNotFoundError(f"Did not find Iserv module : {module_name}")


    def log_out(self):
        print("[*] Logging out")
        self.driver.get(f"{self.base_url}/app/logout")


    def close(self):
        print("[*] Closing browser")
        self.driver.quit()


from .. import Session, Email
from ..Module import Module

from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By


class EmailModule(Module):

    def __init__(self, session : Session):
        super().__init__("E-Mail", session)

    def get_emails(self):
        self.navigate()
        print("[*] Getting email rows")
        email_rows = WebDriverWait(self.content, 3).until(EC.presence_of_all_elements_located((By.CLASS_NAME, "message-row")))
        emails = []
        for i, email in enumerate(email_rows):
            email_obj = Email()
            email_obj.get_basic_data(email, i, self.session.driver, self.session.base_url)
            emails.append(email_obj)
        return emails


from autoIserv.util import download_file_from_iserv

from datetime import datetime
from selenium.common.exceptions import TimeoutException, MoveTargetOutOfBoundsException
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.firefox.webelement import FirefoxWebElement
from selenium.webdriver.firefox.webdriver import WebDriver
from selenium.webdriver.common.action_chains import ActionChains
from urllib.parse import urljoin
import json
import base64
from os import path


class Email:

    def __init__(self):
        self.html_element = None
        self.index = None
        self.driver = None
        self.real_base_url = None

        self.seen = None
        self.by = None

        self.subject = None
        self.date = None

        self.size_unit = None
        self.size = None

        self.raw_text_content = None
        self.html_content = None
        self.attachments = {}


    def get_basic_data(self, html_element : FirefoxWebElement, index: int, web_driver : WebDriver, base_url : str):
        self.html_element = html_element
        self.index = index
        self.driver = web_driver
        self.real_base_url = base_url.split("/")[0]

        classes = html_element.get_attribute("class").split(" ")
        for css_class in classes:
            if css_class == "mail-row-unseen":
                self.seen = False
            elif css_class == "mail-row-seen":
                self.seen = True

        self.by = html_element.find_element_by_class_name("msg-from").text

        subject_elements = WebDriverWait(html_element, 5).until(EC.presence_of_all_elements_located((By.XPATH, "//td[contains(@class, 'msg-subject') and text() != '']")))

        self.subject = subject_elements[index].get_attribute("title")
        if self.subject == '':
            self.subject = subject_elements[index].text

        date_web_objects = WebDriverWait(html_element, 5).until(EC.presence_of_all_elements_located((By.XPATH, "//td[contains(@class, 'msg-date') and text() != '']")))
        self.date = datetime.strptime(date_web_objects[index].text, "%d.%m.%Y %H:%M")

        size_text = html_element.find_element_by_class_name("msg-size").text.lower()
        size_str, self.size_unit = size_text.split(" ")
        self.size = float(size_str.replace(",", ".").replace(" ", ""))

        self.raw_text_content = None
        self.html_content = None
        self.attachments = {}


    def view(self, download_attachments=True):
        print(f"[*] Viewing Email : '{self.subject}'")
        subject_elements = WebDriverWait(self.driver, 5).until(
            EC.presence_of_all_elements_located((By.XPATH, "//td[contains(@class, 'msg-subject') and text() != '']")))
        try:
            ActionChains(self.driver).move_to_element(subject_elements[self.index]).perform()
        except MoveTargetOutOfBoundsException:
            pass
        subject_elements = WebDriverWait(self.driver, 5).until(
            EC.visibility_of_all_elements_located((By.XPATH, "//td[contains(@class, 'msg-subject') and text() != '']")))
        subject_elements[self.index].click()

        body = WebDriverWait(self.driver, 5).until(
            EC.visibility_of_element_located((By.XPATH, "//div[@id='message-body' and text() != '']")))

        message_content = body.find_element_by_id("message-content")
        self.raw_text_content = message_content.text

        try:
            WebDriverWait(self.driver, 4).until(EC.frame_to_be_available_and_switch_to_it((By.CLASS_NAME, "mail-iframe")))
            self.html_content = f'<div id="message-content"><div class="iframe-container">{self.driver.find_element_by_tag_name("body").get_attribute("innerHTML")}</div></div>'
            self.driver.switch_to.default_content()

        except TimeoutException:
            self.html_content = str(message_content.get_attribute('outerHTML').encode('utf-8'), encoding="utf-8")

        if download_attachments:
            message_attachments = body.find_element_by_id("message-attachments")
            attachments = message_attachments.find_elements_by_class_name("attachment")

            for i, attachment in enumerate(attachments):
                drop_down_button = attachment.find_element_by_tag_name("button")
                drop_down_button.click()

                drop_down_menu = attachment.find_element_by_class_name("dropdown-menu")

                download_link = WebDriverWait(drop_down_menu, 5).until(
                    EC.visibility_of_element_located((By.CLASS_NAME, "attachment-download")))

                download_url = urljoin(self.real_base_url, download_link.get_attribute('href'))

                print(f"[*] Downloading Attachment : {download_url}")
                result = download_file_from_iserv(self.driver, download_url)
                if result:
                    print(f"[*] '{result[0]}' was downloaded")
                    self.attachments[result[0]] = result[1].content

        self.driver.find_element_by_id("mail-back").click()


    def __repr__(self):
        return f"<autoIserv.Email.Email (subject='{self.subject}' seen={self.seen} by='{self.by}' size={self.size}{self.size_unit} date={self.date})>"


    def save(self, email_dir, attachment_dir=None, store_attachments_in_file=True):
        data = {
            "Subject" : self.subject,
            "Seen" : self.seen,
            "By" : self.by,
            "Size" : {
                "Value" : self.size,
                "Unit" : self.size_unit
            },
            "Date" : self.date.strftime("%d.%m.%Y %H_%M"),
            "AttachmentNames" : []
        }

        if self.html_content:
            data["raw_html"] = self.html_content

        if self.raw_text_content:
            data["raw_text"] = self.raw_text_content

        if len(self.attachments) > 0:
            if store_attachments_in_file:
                data["Attachments"] = {}
            for attachment_name in self.attachments:
                attachment_content = self.attachments[attachment_name]
                data["AttachmentNames"].append(attachment_name)
                if store_attachments_in_file:
                    data["Attachments"][attachment_name]= str(base64.b64encode(attachment_content), encoding="utf-8")
                if attachment_dir:
                    if path.isdir(attachment_dir):
                        file_path = f"{attachment_dir}\\{attachment_name}"
                        print(f"[*] Saving Attachment to {file_path}")
                        file = open(file_path, mode="wb")
                        file.write(attachment_content)
                        file.close()

        email_path = f"{email_dir}\\" + f"{self.subject}-{self.date}-{self.size}{self.size_unit}_{self.index}.email-json".replace(":", "").replace("\\", "").replace("/", "").replace("?", "").replace("*", "").replace("<", "").replace(">", "").replace('"', "").replace("|", "")
        print(f"[*] Saving email to : {email_path}")
        with open(email_path, 'w', encoding="utf-8") as file:
            json.dump(data, file)


    def save_attachment_to_file(self, attachment, attachment_dir):
        file_path = f"{attachment_dir}\\{attachment_dir}"
        file = open(file_path, mode="wb")
        file.write(self.attachments[attachment])
        file.close()


    @staticmethod
    def load(path, store_attachments=True):
        data = json.load(open(path, mode="r", encoding="utf-8"))
        email = Email()
        email.subject = data["Subject"]
        email.seen = data["Seen"]
        email.by = data["By"]
        email.size = data["Size"]["Value"]
        email.size_unit = data["Size"]["Unit"]
        email.date = datetime.strptime(data["Date"], "%d.%m.%Y %H_%M")

        if store_attachments:
            try:
                for attachment in data["Attachments"]:
                    email.attachments[attachment] = base64.b64decode(data["Attachments"][attachment])
            except KeyError:
                pass

        if len(email.attachments) == 0:
            for name in data["AttachmentNames"]:
                email.attachments[name] = b""

        if data["raw_html"]:
            email.html_content = data["raw_html"]

        if data["raw_text"]:
            email.raw_text_content = data["raw_text"]

        return email


import pathlib
from setuptools import setup, find_packages

packages = list(find_packages())
HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()


setup(
    name="auto-iserv",
    version="0.1.8",
    description="A Library that can be used to create tools to automate Iserv without an API key",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/RedstoneMedia/auto-iserv",
    author="RedstoneMedia",
    keywords="Iserv automation library auto",
    license="GNU General Public License v3.0",
    packages=packages + ["autoIservCredGen"],
    python_requires='>=3.8',
    include_package_data=True,
    install_requires=["PyYAML>=5.3.1", "selenium>=3.141.0", "requests>=2.23.0", "urllib3>=1.25.8", "pycryptodomex>=3.9.8"],
    entry_points={
        "console_scripts": [
            "gen-iserv-credential=autoIservCredGen.__main__:main",
        ]
    },
)


from autoIserv.login import *
from autoIserv.Session import Session
from autoIserv.Email import Email
from autoIserv.Exercise import Exercise
from autoIserv.Module import Module
from autoIserv import Modules